/**         @file  memdbg.h
 *         @brief  Memory leak tracker header
 *          @date  25/09/2006
 *        @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *     @copyright  Copyright (c) 1997-2024, François Cerbelle
 *
 *  Originally inspired by "L'art du code", Steve Maguire, Microsoft Press
 *
 *   @internal
 *       Compiler  gcc
 *  Last modified  2024-08-03 10:29
 *   Organization  Cerbelle.net
 *        Company  Home
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __MEMDBG_H__
#define __MEMDBG_H__

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "memtrack.h"

#ifdef __cplusplus
extern "C" {
#endif

/** Malloc compatible standard allocation
 *
 *  @param [in] Size : Requested size in bytes
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *
 *  @return Allocated block address
 *      @retval Address if succeeded,
 *      @retval NULL if not possible.
 *
 */
void *dbg_malloc(const size_t Size,
                 const char *File,
                 const int Line,
                 const char *CompilDate,
                 const char *CompilTime,
                 const char *Function);

/** Free compatible standard memory release
 *
 * If the Ptr value is not NULL, try to untrack it. If it can not be found in
 * the tracked list, because it was not tracked, it was allocated by a non
 * monitored function, the function abort the process for investigation
 * (missing a monitoring macro/function in the tracker, bug in the tracker,
 * allocation from an external non-instrumented library).
 * If the Ptr value was tracked, found and removed from the list successfully,
 * forward it to free for actual free.
 *
 *  @param [in] Ptr  : Pointer on the memory to free
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *
 */
void dbg_free(void *Ptr,
              const char *File,
              const int Line,
              const char *CompilDate,
              const char *CompilTime,
              const char *Function);

/** Allocate a table of item from the size of each and number
 *
 *  Uses malloc to allocate and track the memory bloc. If allocation and
 *  tracking succeed, fill the memory block with zeros.
 *
 *  @param [in] NMemb : Item number in the table
 *  @param [in] Size : Item size in bytes
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *
 *  @return Allocated block address
 *      @retval Address if succeeded,
 *      @retval NULL if not possible.
 *
 */
void *dbg_calloc(const size_t NMemb,
                 const size_t Size,
                 const char *File,
                 const int Line,
                 const char *CompilDate,
                 const char *CompilTime,
                 const char *Function);

/** Resize an already allocated and tracked block
 *
 * This function always uses malloc to allocate a new block and force an address
 * change, and a data loss in case of shrink, which is the worst case scenario
 * for realloc.
 *
 * As for realloc, a NULL source pointer makes realloc act as malloc, and a new
 * size set to 0 acts as free. The input pointer is left untouched but should
 * not be used or freed anymore. It is always different than the return value.
 * The input values referenced by the input pointer are not wiped.
 *
 * This implementation can not be used if realloc is used to reduce a huge bloc
 * in order to manage an OOM situation.  Real realloc can succeed by actually
 * downsizing the same memory block, inplace, but this implementation will fail
 * because it first allocate a new block.
 *
 *  @param [in] Ptr  : Pointer on the memory to resize
 *  @param [in] Size : New size in bytes
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *
 *  @return Resized block address
 *      @retval Address if succeeded,
 *      @retval NULL if not possible.
 */
void *dbg_realloc(void *Ptr,
                  const size_t Size,
                  const char *File,
                  const int Line,
                  const char *CompilDate,
                  const char *CompilTime,
                  const char *Function);

/** String duplication with allocation
 *
 *  @param [in] Ptr  : Pointer on the string to copy
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *
 *  @return Copied string address
 *      @retval Address if succeeded,
 *      @retval NULL if not possible.
 *
 */
char *dbg_strdup(const char *Ptr,
                 const char *File,
                 const int Line,
                 const char *CompilDate,
                 const char *CompilTime,
                 const char *Function);

/** Build a formatted string with allocation
 *
 *  @param [inout] p_Ptr  : Pointer on the to be built string
 *  @param [in] p_Format  : Pointer on the format string
 *  @param [in] File : Source file
 *  @param [in] Line : Source line number
 *  @param [in] CompilDate : File compilation date
 *  @param [in] CompilTime : File compilation time
 *  @param [in] Function : Source function name
 *  @param [in] ... : Values referenced by the format string
 *
 *  @return Status of the formatting
 *      @retval >=0 number of chars in the output string (as strdup)
 *      @retval -1 in case of error (*p_Ptr contents is undefined)
 *
 */
int dbg_asprintf(char **p_Ptr,
                 const char *p_Format,
                 const char *File,
                 const int Line,
                 const char *CompilDate,
                 const char *CompilTime,
                 const char *Function,
                 ...);

#ifdef __cplusplus
}
#endif

#endif                          /* __MEMDBG_H__ */

